@@ -11,9 +11,6 @@
 #include "options.h"
 #include "pv.h"
 
-/* We do not set this because it breaks "dd" - see below. */
-/* #undef MAKE_OUTPUT_NONBLOCKING */
-
 #include <stdio.h>
 #include <string.h>
 #include <errno.h>
@@ -194,27 +191,6 @@ int main(int argc, char **argv)
 		pv_state_inputfiles(state, opts->argc, (const char **) (opts->argv));
 	}
 
-	/* Total size calculation, in normal transfer mode. */
-	if (0 == opts->watch_pid) {
-		/*
-		 * If no size was given, try to calculate the total size.
-		 */
-		if (0 == opts->size) {
-			pv_state_linemode_set(state, opts->linemode);
-			pv_state_null_terminated_lines_set(state, opts->null_terminated_lines);
-			opts->size = pv_calc_total_size(state);
-			debug("%s: %llu", "no size given - calculated", opts->size);
-		}
-
-		/*
-		 * If the size is unknown, we cannot have an ETA.
-		 */
-		if (opts->size < 1) {
-			opts->eta = false;
-			debug("%s", "size unknown - ETA disabled");
-		}
-	}
-
 	/*
 	 * If stderr is not a terminal and we're neither forcing output nor
 	 * outputting numerically, we will have nothing to display at all.
@@ -265,43 +241,10 @@ int main(int argc, char **argv)
 		opts->interval = 600;
 
 	/*
-	 * Copy parameters from options into main state.
-	 */
-	pv_state_interval_set(state, opts->interval);
-	pv_state_width_set(state, opts->width, opts->width_set_manually);
-	pv_state_height_set(state, opts->height, opts->height_set_manually);
-	pv_state_no_display_set(state, opts->no_display);
-	pv_state_force_set(state, opts->force);
-	pv_state_cursor_set(state, opts->cursor);
-	pv_state_numeric_set(state, opts->numeric);
-	pv_state_wait_set(state, opts->wait);
-	pv_state_delay_start_set(state, opts->delay_start);
-	pv_state_linemode_set(state, opts->linemode);
-	pv_state_bits_set(state, opts->bits);
-	pv_state_decimal_units_set(state, opts->decimal_units);
-	pv_state_null_terminated_lines_set(state, opts->null_terminated_lines);
-	pv_state_skip_errors_set(state, opts->skip_errors);
-	pv_state_error_skip_block_set(state, opts->error_skip_block);
-	pv_state_stop_at_size_set(state, opts->stop_at_size);
-	pv_state_sync_after_write_set(state, opts->sync_after_write);
-	pv_state_direct_io_set(state, opts->direct_io);
-	pv_state_discard_input_set(state, opts->discard_input);
-	pv_state_rate_limit_set(state, opts->rate_limit);
-	pv_state_target_buffer_size_set(state, opts->buffer_size);
-	pv_state_no_splice_set(state, opts->no_splice);
-	pv_state_size_set(state, opts->size);
-	pv_state_name_set(state, opts->name);
-	pv_state_format_string_set(state, opts->format);
-	pv_state_watch_pid_set(state, opts->watch_pid);
-	pv_state_watch_fd_set(state, opts->watch_fd);
-	pv_state_average_rate_window_set(state, opts->average_rate_window);
-
-	pv_state_set_format(state, opts->progress, opts->timer, opts->eta,
-			    opts->fineta, opts->rate, opts->average_rate,
-			    opts->bytes, opts->bufpercent, opts->lastwritten, opts->name);
-
-	/*
-	 * Set output file, treating no output or "-" as stdout
+	 * Set output file, treating no output or "-" as stdout; we have to
+	 * do this before looking at setting the size, as the size
+	 * calculation looks at the output file if the input size can't be
+	 * calculated (issue #91).
 	 */
 	if (NULL == opts->output || 0 == strcmp(opts->output, "-")) {
 		pv_state_output_set(state, STDOUT_FILENO, "(stdout)");
@@ -323,15 +266,71 @@ int main(int argc, char **argv)
 		pv_state_output_set(state, fd, opts->output);
 	}
 
-#ifdef MAKE_OUTPUT_NONBLOCKING
 	/*
-	 * Try and make the output use non-blocking I/O.
-	 *
-	 * Note that this can cause problems with (broken) applications
-	 * such as dd when used in a pipeline.
+	 * Copy the "stop at size" option before checking the total size,
+	 * since calculating the size from the output block device size
+	 * after this may want to force this setting on, and if we set it
+	 * afterwards, we undo the override.
 	 */
-	fcntl(state->control.output_fd, F_SETFL, O_NONBLOCK | fcntl(state->control.output_fd, F_GETFL));
-#endif				/* MAKE_OUTPUT_NONBLOCKING */
+	pv_state_stop_at_size_set(state, opts->stop_at_size);
+
+	/* Total size calculation, in normal transfer mode. */
+	if (0 == opts->watch_pid) {
+		/*
+		 * If no size was given, try to calculate the total size.
+		 */
+		if (0 == opts->size) {
+			pv_state_linemode_set(state, opts->linemode);
+			pv_state_null_terminated_lines_set(state, opts->null_terminated_lines);
+			opts->size = pv_calc_total_size(state);
+			debug("%s: %llu", "no size given - calculated", opts->size);
+		}
+
+		/*
+		 * If the size is unknown, we cannot have an ETA.
+		 */
+		if (opts->size < 1) {
+			opts->eta = false;
+			debug("%s", "size unknown - ETA disabled");
+		}
+	}
+
+	/*
+	 * Copy the remaining parameters from the options into the main
+	 * state.
+	 */
+
+	pv_state_interval_set(state, opts->interval);
+	pv_state_width_set(state, opts->width, opts->width_set_manually);
+	pv_state_height_set(state, opts->height, opts->height_set_manually);
+	pv_state_no_display_set(state, opts->no_display);
+	pv_state_force_set(state, opts->force);
+	pv_state_cursor_set(state, opts->cursor);
+	pv_state_numeric_set(state, opts->numeric);
+	pv_state_wait_set(state, opts->wait);
+	pv_state_delay_start_set(state, opts->delay_start);
+	pv_state_linemode_set(state, opts->linemode);
+	pv_state_bits_set(state, opts->bits);
+	pv_state_decimal_units_set(state, opts->decimal_units);
+	pv_state_null_terminated_lines_set(state, opts->null_terminated_lines);
+	pv_state_skip_errors_set(state, opts->skip_errors);
+	pv_state_error_skip_block_set(state, opts->error_skip_block);
+	pv_state_sync_after_write_set(state, opts->sync_after_write);
+	pv_state_direct_io_set(state, opts->direct_io);
+	pv_state_discard_input_set(state, opts->discard_input);
+	pv_state_rate_limit_set(state, opts->rate_limit);
+	pv_state_target_buffer_size_set(state, opts->buffer_size);
+	pv_state_no_splice_set(state, opts->no_splice);
+	pv_state_size_set(state, opts->size);
+	pv_state_name_set(state, opts->name);
+	pv_state_format_string_set(state, opts->format);
+	pv_state_watch_pid_set(state, opts->watch_pid);
+	pv_state_watch_fd_set(state, opts->watch_fd);
+	pv_state_average_rate_window_set(state, opts->average_rate_window);
+
+	pv_state_set_format(state, opts->progress, opts->timer, opts->eta,
+			    opts->fineta, opts->rate, opts->average_rate,
+			    opts->bytes, opts->bufpercent, opts->lastwritten, opts->name);
 
 	/* Initialise the signal handling. */
 	pv_sig_init(state);
