@@ -181,6 +181,12 @@ static ssize_t pv__transfer_read_repeated(int fd, void *buf, size_t count)
  * see if we can write any more, and keep trying, to make sure we empty the
  * buffer as much as we can.
  *
+ * While this is called after a successful write-possible select(), write() is
+ * not guaranteed to succeed for _all_ sizes; we may end up returning 0 if this
+ * occurs. (The first write() may return -1 / EINTR if the consumer doesn't
+ * read any data before our timeout and the buffer of whatever stdout is is
+ * near-full.) (see https://codeberg.org/a-j-wood/pv/pulls/93)
+ *
  * If "sync_after_write" is true, we call fdatasync() after each write() (or
  * fsync() if _POSIX_SYNCHRONIZED_IO is not > 0).
  *
@@ -686,7 +692,11 @@ static int pv__transfer_write(pvstate_t state, bool *eof_in, bool *eof_out, long
 						       state->transfer.write_position,
 						       (size_t) (state->transfer.to_write),
 						       state->control.sync_after_write);
-		debug("%s: %ld", "bytes written", (long) nwritten);
+		if (nwritten < 0) {
+			debug("%s: %ld: %s", "bytes written", (long) nwritten, strerror(errno));
+		} else {
+			debug("%s: %ld", "bytes written", (long) nwritten);
+		}
 #if HAVE_SETITIMER
 		memset(&new_timer, 0, sizeof(new_timer));
 		new_timer.it_interval.tv_sec = 0;
@@ -704,13 +714,7 @@ static int pv__transfer_write(pvstate_t state, bool *eof_in, bool *eof_out, long
 #endif				/* HAVE_SETITIMER */
 	}
 
-	if (0 == nwritten) {
-		/*
-		 * Write returned 0 - EOF on output.
-		 */
-		*eof_out = true;
-		return 1;
-	} else if (nwritten > 0) {
+	if (nwritten > 0) {
 		/*
 		 * Write returned >0 - data successfully written.
 		 */
@@ -796,15 +800,20 @@ static int pv__transfer_write(pvstate_t state, bool *eof_in, bool *eof_out, long
 	}
 
 	/*
-	 * If we reach this point, nwritten<0, so there was an error.
+	 * If we reach this point, nwritten<=0, so there may be an error.
 	 */
 
 	/*
-	 * If a write error occurred but it was EINTR or EAGAIN, just wait a
-	 * bit and then return zero, since this was a transient error.
+	 * If a write error occurred but it was EINTR or EAGAIN, or write(2)
+	 * blocked on first write such that nwritten == 0, just wait a bit and
+	 * then return zero, since this was a transient error.
 	 */
-	if ((EINTR == errno) || (EAGAIN == errno)) {
-		debug("%s: %s", "transient write error - waiting briefly", strerror(errno));
+	if ((0 == nwritten) || (EINTR == errno) || (EAGAIN == errno)) {
+		if (0 == nwritten) {
+			debug("%s", "attempted write blocked - waiting briefly");
+		} else {
+			debug("%s: %s", "transient write error - waiting briefly", strerror(errno));
+		}
 		(void) is_data_ready(-1, NULL, -1, NULL, 10000);
 		return 0;
 	}
